package weblech.spider;

import atomicsets.OrderedLock;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class SpiderRunnable implements Runnable, Constants {

	private SpiderConfig config;
	private Spider spider;
	private UDorSWrapper udors;
	
    /** Whether the spider should quit */
    private boolean quit;
    /** Count of running Spider threads. */
    private int running;
    
	public SpiderRunnable(SpiderConfig config) {
		this.config = config;
		spider = new Spider(config, new OrderedLock());
		udors = new UDorSWrapper(config, new OrderedLock());
	}
	
    public void start()
    {
        quit = false;
        running = 0;

        for(int i = 0; i < config.getSpiderThreads(); i++)
        {
           spider.setup();
            Thread t = new Thread(this, "Spider-Thread-" + (i + 1));
            t.start();
            running++;
        }
    }

    public void stop()
    {
        quit = true;
    }

    public boolean isRunning()
    {
        return running == 0;
    }
	
	public void readCheckpoint() {
		spider.readCheckpoint();
	}
	
	
	public void run() {
	       HTMLParser htmlParser = new HTMLParser(config);
	        URLGetter urlGetter = new URLGetter(config);

	        while((spider.queueSize() > 0 || spider.downloadsInProgress()) && quit == false)
	        {
	            spider.checkpointIfNeeded();
	            if(spider.queueSize() == 0 && spider.downloadsInProgress())
	            {
	                // Wait for a download to finish before seeing if this thread should stop
	                try
	                {
	                    Thread.sleep(QUEUE_CHECK_INTERVAL);
	                }
	                catch(InterruptedException ignored)
	                {
	                }
	                // Have another go at the loop
	                continue;
	            }
	            else if(spider.queueSize() == 0)
	            {
	                break;
	            }
	            URLToDownload nextURL = spider.getNextURL();
	            int newDepth = nextURL.getDepth() + 1;
	            int maxDepth = config.getMaxDepth();
	            List newURLs = spider.downloadURL(nextURL, urlGetter, htmlParser);

	            newURLs = udors.filterURLs(newURLs);

	            ArrayList u2dsToQueue = new ArrayList();
	            for(Iterator i = newURLs.iterator(); i.hasNext(); )
	            {
	                URL u = (URL) i.next();
	                // Download if not yet downloaded, and the new depth is less than the maximum
	                udors.scheduleURL(nextURL, newDepth, maxDepth, u2dsToQueue, u);
	            }
	            spider.queueURLs(u2dsToQueue);
	        }
	        spider.cleanup();
	        running--;
	}
	
}
